/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.dataserver;

import java.sql.*;
import java.util.*;
import java.io.IOException;
import java.io.PrintWriter;
import org.w3c.dom.*;
import de.hendriklipka.ldse.common.*;
import de.hendriklipka.ldse.server.data.document.*;
import de.hendriklipka.ldse.server.data.server.*;

import jdbm.*;

public class JDBMSDBConnection implements de.hendriklipka.ldse.server.dataserver.DBConnection
{
    // Objekte fuer das Singleton
    protected static JDBMSDBConnection   _engine=null;
    protected static Object         _syncObj=new Object();

    protected JDBMRecordManager _manager=null;
    protected JDBMHashtable _DocsTable=null; // docs by ID 
    protected JDBMHashtable _TripletDocsTable=null; // docs by triplet (a vector per triplet)
    protected JDBMHashtable _WordsTable=null; // words by ID
    protected JDBMHashtable _TripletWordsTable=null; // words by triplets (a vector per triplet)
    protected JDBMHashtable _WordDocumentsTable=null; // docs by word (a vector per word)
    
    public JDBMSDBConnection(Node node)
    {
        try
        {
            _manager=new JDBMRecordManager("ldse");
            _DocsTable=_manager.getHashtable("docs");
            _TripletDocsTable=_manager.getHashtable("tripletdocs");
            _WordsTable=_manager.getHashtable("words");
            _TripletWordsTable=_manager.getHashtable("tripletwords");
            _WordDocumentsTable=_manager.getHashtable("worddocuments");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }
    
    public static JDBMSDBConnection openConnection(Node node)
    {
        try
        {
            if (null==_engine)
            {
                synchronized (_syncObj)
                {
                    if (null==_engine)
                    {
                        _engine=new JDBMSDBConnection(node);
                    }
                }
            }

            return _engine;
        }
        catch (Exception ex)
        {
            ex.printStackTrace(Log.getWriter("database","error",0));

            return null;
        }
    }

    public void writeStatistic(PrintWriter pw) throws IOException
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
    }

    public void update(Node configNode)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
    }

    public String unmarkDocument(String name)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String sweepSpider(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String shutdown()
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public ServerDescriptor[] queryServers(TripletList have)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public DocumentDescriptor[] queryDocuments(TripletList have, TripletList info)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String putServerDescriptor(ServerDescriptor sd)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String putDocument(String name, DocumentDescriptor dd, WordDocument words[])
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String markSpider(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public void initDataBase(String driverclass, String url, String user, String pwd)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
    }

    public WordData[] getWordsForTriplets(TripletList list)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public int getTripletCount(int tripletNum)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return 0;
    }

    public int[] getSubServers()
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public ServerDescriptor getServerDescriptor(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public DocumentDescriptor getDocumentDescriptor(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public DocumentDescriptor getDocumentDescriptor(String name)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String[] getAllForSpider(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public void fillWordData(WordData wordData)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
    }

    public boolean existDocument(String name)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return false;
    }

    public String deleteSubServer(String name)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String deleteSubServer(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String deleteSpider(int id)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String delete(String name)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public ServerDescriptor createServerDescriptor(int id, String name, int port)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public ServerDescriptor createLocalServerDescriptor(int id, String name, int port)
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

    public String cleanDatabase()
    {
        // This method is derived from interface de.hendriklipka.ldse.server.dataserver.DBConnection
        // to do: code goes here
        return null;
    }

}