/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.common;


import java.io.Reader;
import java.util.Vector;

import org.xml.sax.*;

import com.microstar.xml.*;

import de.hendriklipka.ldse.common.ResultDescriptor;

/**
  Liest ein XML-Dokument aus einem Reader aus. Es wird erwartet, dass dieses Dokument der
  DTD in "query.dtd" entspricht. Das Parsen wird mit einer State-Machine erledigt.
  Die Ergebnisse werden ueber CallBacks zuruckgegeben (an ResultListener)
*/
public class ResultParser implements DocumentHandler, ErrorHandler
{
  protected Vector _listener=new Vector(5,5);
  protected Reader _reader=null;

  protected Parser parser=null;	
  protected InputSource in=null;
  
  protected ResultDescriptor _actualRD=null;
  protected int _state=START;
  protected final static int START=0;
  protected final static int IN_RESULT=10;
  protected final static int FINISHED=11;

  protected final static int IN_ANSWER=20;
  protected final static int IN_URL=21;
  protected final static int IN_RELEVANCE=22;
  protected final static int IN_TITLE=23;
  protected final static int IN_EXCERPT=24;
  protected final static int IN_SIZE=25;
  protected final static int IN_LASTCHANGE=26;
  protected final static int IN_TYPE=27;

  protected final static int IN_DATA=30;
  
  public void init(Reader r)
  {
    _reader=r;
       
    try 
    {	    
      in=new InputSource(_reader);
      parser=new SAXDriver();
      parser.setDocumentHandler(this);
      parser.setErrorHandler(this);
    } 
    catch (Exception ex) 
    {	    
      ex.printStackTrace();
    }	
  }
  
  public void readResult() throws Throwable
  {
//    try
//    {
      // Listener vom Start benachrichtigen
      //ToDo: in startDocument machen?
      for (int i=0;i<_listener.size();i++)
      {
        ResultListener rl=(ResultListener)_listener.elementAt(i);
        try
        {
          rl.startResult();
        }
        catch (Exception ex)
        {
          ex.printStackTrace();
        }
      }

      // Ergebnisse lesen
      parser.parse(in);	

      // Listener vom Ende benachrichtigen
      //ToDo: in endDocument machen?
      
      for (int i=0;i<_listener.size();i++)
      {
        ResultListener rl=(ResultListener)_listener.elementAt(i);
        try
        {
          rl.endResult();
        }
        catch (Exception ex)
        {
          ex.printStackTrace();
        }
      }
      /*
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
    */
  }

  protected void sendResult(ResultDescriptor rd)
  {
    for (int i=0;i<_listener.size();i++)
    {
      ResultListener rl=(ResultListener)_listener.elementAt(i);
      rl.addResult(rd);
    }
  }
  
  public void addResultListener(ResultListener rl)
  {
    _listener.addElement(rl);
  }
  
  public void setDocumentLocator (Locator locator)
  {
  }

  public void startDocument () throws SAXException
  {
  }

  public void endDocument () throws SAXException
  {
  }

  public void startElement (String name, AttributeList atts) throws SAXException
  {
    if(name.equals("answer"))
    {
      _state=IN_ANSWER;
      _actualRD=new ResultDescriptor();
    }
    else if(name.equals("data"))
    {
      _state=IN_DATA;
    }
    else if(name.equals("url"))
    {
      _state=IN_URL;
    }
    else if(name.equals("relevance"))
    {
      _state=IN_RELEVANCE;
    }
    else if(name.equals("title"))
    {
      _state=IN_TITLE;
    }
    else if(name.equals("excerpt"))
    {
      _state=IN_EXCERPT;
    }
    else if(name.equals("size"))
    {
      _state=IN_SIZE;
    }
    else if(name.equals("lastchange"))
    {
      _state=IN_LASTCHANGE;
    }
    else if(name.equals("queryresult"))
    {
      String s=atts.getValue("count");
      for (int i=0;i<_listener.size();i++)
      {
        ResultListener rl=(ResultListener)_listener.elementAt(i);
        rl.setResultNum(Integer.parseInt(s));
      }
      _state=IN_RESULT;
    }
  }

  public void endElement (String name) throws SAXException
  {
    switch(_state)
    {
      case IN_RESULT:
        _state=FINISHED;
        break;
      case IN_URL:
        _state=IN_ANSWER;
        break;
      case IN_RELEVANCE:
        _state=IN_ANSWER;
        break;
      case IN_TITLE:
        _state=IN_ANSWER;
        break;
      case IN_EXCERPT:
        _state=IN_ANSWER;
        break;
      case IN_SIZE:
        _state=IN_ANSWER;
        break;
      case IN_TYPE:
        _state=IN_ANSWER;
        break;
      case IN_LASTCHANGE:
        _state=IN_ANSWER;
        break;
      case IN_ANSWER:
        if(name.equals("answer"))
        {
          _state=IN_RESULT;
          // ResultDescriptor senden
          sendResult(_actualRD);
        }
        break;
      case IN_DATA:
        _state=IN_RESULT;
        break;
    }
  }

  public void characters (char ch[], int start, int length) throws SAXException
  {
    switch(_state)
    {
      case IN_URL:
        _actualRD.setURL(new String(ch,start,length));
        break;
      case IN_RELEVANCE:
        try
        {
          double d=(Double.valueOf(new String(ch,start,length))).doubleValue();
          _actualRD.setRelevance(d);
        }
        catch (Exception ex) {}
        break;
      case IN_TITLE:
        _actualRD.setTitle(new String(ch,start,length));
        break;
      case IN_EXCERPT:
        _actualRD.setExcerpt(new String(ch,start,length));
        break;
      case IN_SIZE:
        try
        {
          int i=Integer.parseInt(new String(ch,start,length));
          _actualRD.setSize(i);
        }
        catch (Exception ex) {}
        break;
      case IN_TYPE:
        _actualRD.setType(new String(ch,start,length));
        break;
      case IN_LASTCHANGE:
        _actualRD.setLastChange(new String(ch,start,length));
        break;
    }
  }

  public void ignorableWhitespace (char ch[], int start, int length) throws SAXException
  {
  }

  public void processingInstruction (String target, String data) throws SAXException
  {
  }
  
  // Errors
	public void fatalError (SAXParseException e)	throws SAXParseException	
	{	    
	  throw e;	
	}	
	
  // Errors
	public void error (SAXParseException e)	throws SAXParseException	
	{	    
	  throw e;	
	}	
	
	// Warnings
	public void warning (SAXParseException err)	throws SAXParseException	
	{	    
	}
}