/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.w3c.dom.*;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.query.QueryServer;
import de.hendriklipka.ldse.server.broker.BrokerServer;
import de.hendriklipka.ldse.server.dataserver.DataServer;
import de.hendriklipka.ldse.server.dataserver.*;

public class DocumentServer implements de.hendriklipka.ldse.common.Updateable, de.hendriklipka.ldse.common.StatisticWriter
{
  protected static DBConnection _con=null;
  protected static DataServer _ds=null;
  protected static QueryServer _qs=null;
  protected static BrokerServer _bs=null;
  protected static DocumentServer _docServer=null;
  
  protected DocumentServer()
  {
    try
    {
      // Konfiguration einlesen (bis zur Config des Logwriters...)
      Element el=null;
      try
      {
  	    InputSource	input = Resolver.createInputSource (new File ("documentserver.xml"));
	      XmlDocument	doc = XmlDocument.createXmlDocument (input, false);
        el=doc.getDocumentElement();
        NodeList cnl=el.getElementsByTagName("debug");
        Node configNode=cnl.item(0);
        Log.init(configNode);
      }
      catch (Throwable ex)
      {
        ex.printStackTrace(System.err); // noch kein Log vorhanden...
        System.exit(1);
      }
      
      // Datenbank starten
      NodeList nl=el.getElementsByTagName("database");
      Node node=nl.item(0);
      if (null!=node)
      {
        _con=RDBMSDBConnection.openConnection(node);
      }
 
      // BrokerServer erzeugen
      _bs=new BrokerServer(_con,null);

      // QueryServer starten
      nl=el.getElementsByTagName("queryserver");
      node=nl.item(0);
      int qPort=-1;
      if (null!=node)
      {
        _qs=new QueryServer(_con,node,_bs);
        _qs.start();
        qPort=_qs.getPort();
      } 

      // DataServer starten
      nl=el.getElementsByTagName("dataserver");
      node=nl.item(0);
      if (null!=node)
      {
        _ds=new DataServer(_con,node,this,this,qPort);
        _ds.start();
      }

    }
    catch (Exception ex)
    {
      ex.printStackTrace(Log.getWriter(0));
    }
  }

  public static void main(String[] args)
  {
    _docServer=new DocumentServer();
  }

  // in diesem Fall ist der Node egal, wir verteilen lediglich
  public void update(Node configNode)
  {
    try
    {
      // Konfiguration einlesen (bis zur Config des Logwriters...)
	    InputSource	input = Resolver.createInputSource (new File ("dsconfig.xml"));
	    XmlDocument	doc = XmlDocument.createXmlDocument (input, false);
      Element el=doc.getDocumentElement();
      NodeList nl=el.getElementsByTagName("debug");
      Node node=nl.item(0);
      if (null!=node)
        Log.getInstance().update(node);
      
      nl=el.getElementsByTagName("database");
      node=nl.item(0);
      if ((null!=node)&&(null!=_con))
      {
        _con.update(node);
      }
 
      // DataServer starten
      nl=el.getElementsByTagName("dataserver");
      node=nl.item(0);
      if ((null!=node)&&(null!=_ds))
      {
        _ds.update(node);
      }
      
      // QueryServer starten
      nl=el.getElementsByTagName("queryserver");
      node=nl.item(0);
      if ((null!=node)&&(null!=_qs))
      {
        _qs.update(node);
      } 
      
      // BrokerServer starten
      nl=el.getElementsByTagName("brokerserver");
      node=nl.item(0);
      if ((null!=node)&&(null!=_bs))
      {
        _bs.update(node);
      }
    }
    catch (Exception ex)
    {
      ex.printStackTrace(Log.getWriter(0));
    }
  }
  
  public void writeStatistic(PrintWriter pw) throws IOException
  {
    // Substatistiken
    if (null!=_con)
      _con.writeStatistic(pw);
    if (null!=_bs)
      _bs.writeStatistic(pw);
    if (null!=_qs)
      _qs.writeStatistic(pw);
    if (null!=_ds)
      _ds.writeStatistic(pw);
  }
}