/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.data.server;

import java.util.Properties;

import de.hendriklipka.ldse.server.data.document.Triplet;
import de.hendriklipka.ldse.server.data.document.TripletList;

public class ServerDescriptor
{
  int _id=-1;
  String _name="";
  int _port=-1;
  int _words=0;
  int _triplets=0;
  int _documents=0;
  int _subServers=0;
  int _subDocuments=0;
  int _subTriplets=0;
  int _subWords=0;
  
  TripletList _tripletList=new TripletList();
  
  public ServerDescriptor(int id, String name, int port, int docs, int words, int triplets,
                          int subDocs, int subWords, int subTriplets, int subServers)
  {
    _id=id;
    _name=name;
    _port=port;
    _words=words;
    _triplets=triplets;
    _documents=docs;
    _subServers=subServers;
    _subDocuments=subDocs;
    _subTriplets=subTriplets;
    _subWords=subWords;
  }
  
  public void addTriplet(int id, int num, int docCount, int maxDocNum)
  {
    TripletEntry te=new TripletEntry(id,num,docCount,maxDocNum);
    _tripletList.addTriplet(te);
  }
  
  public TripletList getTripletList()
  {
    return _tripletList;
  }
  
  public int getID()
  {
    return _id;
  }
  
  public String getName()
  {
    return _name;
  }
  
  public int getPort()
  {
    return _port;
  }

  public int getWords()
  {
    return _words;
  }
  
  public int getTriplets()
  {
    return _triplets;
  }

  public int getDocuments()
  {
    return _documents;
  }

  public int getSubServers()
  {
    return _subServers;
  }

  public int getSubDocuments()
  {
    return _subDocuments;
  }

  public int getSubTriplets()
  {
    return _subTriplets;
  }

  public int getSubWords()
  {
    return _subWords;
  }
  
  public void setSubServers(int i)
  {
    _subServers=i;
  }
  
  public void setSubDocs(int i)
  {
    _subDocuments=i;
  }
  
  public void setSubWords(int i)
  {
    _subWords=i;
  }
  
  public void setSubTriplets(int i)
  {
    _subTriplets=i;
  }
  
  public void setID(int i)
  {
    _id=i;
  }
  
  public Properties getData()
  {
    Properties p=new Properties();
    p.put("name",_name);
    p.put("port",Integer.toString(_port));
    p.put("words",Integer.toString(_words));
    p.put("docs",Integer.toString(_documents));
    p.put("triplets",Integer.toString(_triplets));
    p.put("subwords",Integer.toString(_subWords));
    p.put("subdocs",Integer.toString(_subDocuments));
    p.put("subtriplets",Integer.toString(_subTriplets));
    p.put("subservers",Integer.toString(_subServers));
    return p;
  }
  
  public ServerDescriptor(Properties data, int id)
  {
    _id=id;
    String s=data.getProperty("name");
    if (s!=null)
    {
      _name=s;
    }
    s=data.getProperty("port");
    if (s!=null)
    {
      _port=Integer.parseInt(s);
    }
    s=data.getProperty("words");
    if (s!=null)
    {
      _words=Integer.parseInt(s);
    }
    s=data.getProperty("docs");
    if (s!=null)
    {
      _documents=Integer.parseInt(s);
    }
    s=data.getProperty("triplets");
    if (s!=null)
    {
      _triplets=Integer.parseInt(s);
    }
    s=data.getProperty("subwords");
    if (s!=null)
    {
      _subWords=Integer.parseInt(s);
    }
    s=data.getProperty("subdocs");
    if (s!=null)
    {
      _subDocuments=Integer.parseInt(s);
    }
    s=data.getProperty("subtriplets");
    if (s!=null)
    {
      _subTriplets=Integer.parseInt(s);
    }
    s=data.getProperty("subservers");
    if (s!=null)
    {
      _subServers=Integer.parseInt(s);
    }
  }
}