/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.dataserver;

import java.net.*;
import java.io.*;
import java.util.*;

import org.w3c.dom.*;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.data.document.*;
import de.hendriklipka.ldse.server.data.server.ServerDescriptor;
import de.hendriklipka.ldse.server.data.server.TripletEntry;

public class DataClient
{
  protected String _server=null;
  protected int _port=0;
  
  protected Socket _socket=null;
  protected InputStream _istream=null;
  protected OutputStream _ostream=null;
  protected PrintWriter _writer=null;
  protected BufferedReader _reader=null;

  // dieser Konstruktor wird vom DataServer verwendet
  // da dieser keinen ConfigNode hat...
  public DataClient(String server, int port)
  {
    // Verbindung oeffnen
    _server=server;
    _port=port;
    try
    {
      _socket=new Socket(_server,_port);
      _istream=_socket.getInputStream();
      _ostream=_socket.getOutputStream();
      _reader=new BufferedReader(new InputStreamReader(_istream),1);
      _writer=new PrintWriter(new OutputStreamWriter(_ostream));
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
  }
  
  public DataClient(Node configNode)
  {
    String serverName="127.0.0.1";
    String port="7623";
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("port"))
      {
        port=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
      else if (nn.equals("address"))
      {
        serverName=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
    }
    _server=serverName;
    _port=Integer.parseInt(port);
    try
    {
      _socket=new Socket(serverName,_port);
      _istream=_socket.getInputStream();
      _ostream=_socket.getOutputStream();
      _reader=new BufferedReader(new InputStreamReader(_istream),1);
      _writer=new PrintWriter(new OutputStreamWriter(_ostream));
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
  }

  // fuer jedes Kommando eine eigene Funktion:
  // Kommando senden
  // evt. Parameter uebertragen
  // Ergebnis lesen
  
  public synchronized boolean existDocument(URL url)
  {
    Log.log("dataclient","command",0,"DataClient.existDocument");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("exist<>"+url.toString());
      _writer.flush();
      String answer=_reader.readLine();
      Log.log("dataclient","command",1,"DataClient.existDocument - returning");
      return (answer.equals("yes"));
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return false;
  }
  
  public synchronized boolean unmarkDocument(DocumentDescriptor desc)
  {
    Log.log("dataclient","command",0,"DataClient.unmark");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      Log.log("dataclient","command",2,"DataClient.unmark");
      // Befehl senden
      _writer.println("unmark<>"+desc.getURL().toString());
      _writer.flush();
      // Antwort lesen
      String answer=_reader.readLine();
      if (answer.startsWith("ok"))
      { 
        return true;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return false;
  }
  
  public synchronized boolean putDocument(DocumentDescriptor desc, WordDocument[] words)
  {
    Log.log("dataclient","command",0,"DataClient.putDocument");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      Log.log("dataclient","command",2,"DataClient.putDocument1");
      // Befehl senden
      _writer.println("put<>"+desc.getURL().toString());
      // DocumentDescriptor senden
      Log.log("dataclient","command",2,"DataClient.putDocument2");
      Properties props=desc.getData();
      Enumeration keys=props.keys();
      Log.log("dataclient","command",2,"DataClient.putDocument3");
      while (keys.hasMoreElements())
      {
        String name=(String)keys.nextElement();
        String value=props.getProperty(name);
        _writer.println(name+"="+value);
        _writer.flush();
        Log.log("dataclient","command",2,"put property: "+name+"="+value);
      }
      Log.log("dataclient","command",2,"DataClient.putDocument4");
      _writer.println("<>");
      _writer.flush();
      // Daten der Triplets senden
      Enumeration e=desc.getList().getTriplets();
      Log.log("dataclient","command",2,"DataClient.putDocument5");
      while (e.hasMoreElements())
      {
        Triplet tr=(Triplet)e.nextElement();
        _writer.println(tr.num+"="+tr.count);
        _writer.flush();
      }
      _writer.println("<>");
      _writer.flush();
      for (int i=0;i<words.length;i++)
      {
        WordDocument wd=words[i];
        _writer.print(wd._word._word+"="+wd._count+"=");
        e=wd._word._triplets.getTriplets();
        while (e.hasMoreElements())
        {
            Triplet tr=(Triplet)e.nextElement();
            for (int k=0;k<tr.count;k++)
            {
                _writer.print(""+tr.num+",");
            }
        }
        _writer.println();
      }
      _writer.println("<><>");
      _writer.flush();
      Log.log("dataclient","command",2,"DataClient.putDocument6");
      // Antwort lesen
      String answer=_reader.readLine();
      Log.log("dataclient","command",1,"DataClient.putDocument - returning with "+answer);
      if (answer.startsWith("ok"))
      { 
        return true;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return false;
  }
  
  public synchronized DocumentDescriptor getDocument(URL url)
  {
    Log.log("dataclient","command",0,"DataClient.getDocument");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("get<>"+url.toString());  
      _writer.flush();
      String answer=_reader.readLine();  
      if (answer.startsWith("ok"))
      { // Daten einlesen
        Properties props=new Properties();
        answer=_reader.readLine();
        while (true)
        {
          if (answer.equals("<><>"))
            break;
          int pos=answer.indexOf("=");
          if (-1==pos)
            continue;
          String name=answer.substring(0,pos);
          String val=answer.substring(pos+1);
          Log.log("dataclient","command",2,"got property: "+name+"="+val);
          props.put(name,val);
          answer=_reader.readLine();
        }
        Log.log("dataclient","command",1,"DataClient.getDocument - returning");
        return new DocumentDescriptor(props);
      }
      else
      {
        Log.log("dataclient","command",1,"DataClient.getDocument - returning with error: "+answer);
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return null;
  }

  public synchronized void close()
  {
    try
    {
      _writer.println("close");
      _writer.flush();
      _writer.close();
      _reader.close();
      _istream.close();
      _ostream.close();
      _socket.close();
    }
    catch (IOException ex) {}
  }
  
  public synchronized void shutdown()
  {
    Log.log("dataclient","command",0,"shutdown");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("admin<>passwd");
      _writer.flush();
      String line=_reader.readLine();
      if (!line.equals("ok"))
        return;
      _writer.println("shutdown");
      _writer.flush();
      line=_reader.readLine();
      _writer.flush();
      if (!line.equals("ok"))
        return;
      _writer.close();
      _reader.close();
      _istream.close();
      _ostream.close();
      _socket.close();
    }
    catch (IOException ex) {}
  }
  
  public synchronized boolean delete(URL url)
  {
    Log.log("dataclient","command",0,"DataClient.delete");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("deld<>"+url.toString());
      _writer.flush();
      String answer=_reader.readLine();
      if (answer.startsWith("ok"))
      { 
        return true;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return false;
  }
  
  public synchronized DocumentDescriptor[] getAllForSpider(int id)
  {
    Log.log("dataclient","command",0,"DataClient.getAllForSpider");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("sget<>"+id);
      _writer.flush();
      String answer=_reader.readLine();
      if (answer.startsWith("ok"))
      {
        Vector v=new Vector(10,10);
        answer=_reader.readLine();
        while (true)
        {
          if (answer.equals("<><>"))
            break;
          if (null==answer)
            continue;
          v.addElement(answer);
          answer=_reader.readLine();
        }
        DocumentDescriptor[] dsa=new DocumentDescriptor[v.size()];
        for (int i=0;i<dsa.length;i++)
        {
          DocumentDescriptor ds=getDocument(new URL((String)v.elementAt(i)));
          dsa[i]=ds;
        }
        return dsa;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return null;
  }
  
  public synchronized boolean markSpider(int id)
  {
    Log.log("dataclient","command",0,"DataClient.markSpider");
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("mark<>"+id);
      _writer.flush();
      String answer=_reader.readLine();
      if (answer.startsWith("ok"))
      { 
        return true;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return false;
  }
  
  public synchronized boolean sweepSpider(int id)
  {
    Log.log("dataclient","command",0,"DataClient.sweepSpider for spider "+id);
    try
    {
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      _writer.println("sweep<>"+id);
      _writer.flush();
      String answer=_reader.readLine();
      if (null==answer)
      {
        Log.log("dataclient","command",0,"DataClient.sweepSpider: null!");
        return false;
      }
      if (answer.startsWith("ok"))
      { 
        return true;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
    return false;
  }
  
  public boolean putServerDescriptor(ServerDescriptor sd)
  {
    try
    {
      Log.log("dataclient","command",1,"DataClient.putServerDescriptor");
      // Stream leeren
      while (_reader.ready())
        _reader.read();
      // Befehl senden
      _writer.println("puts<>"+sd.getID());
      _writer.flush();
      // ServerDescriptor senden
      Properties props=sd.getData();
      Enumeration keys=props.keys();
      while (keys.hasMoreElements())
      {
        String name=(String)keys.nextElement();
        String value=props.getProperty(name);
        _writer.println(name+"="+value);
        Log.log("dataclient","command",2,"put property: "+name+"="+value);
      _writer.flush();
      }
      _writer.println("<>");
      Log.log("dataclient","command",1,"DataClient.putServerDescriptor: sent all properties");
      // Daten der Triplets senden
      Enumeration e=sd.getTripletList().getTriplets();
      while (e.hasMoreElements())
      {
        TripletEntry tr=(TripletEntry)e.nextElement();
        _writer.println(tr.num+"="+tr.count+"="+tr.getDocCount()+"="+tr.getMaxDocNum());
      }
      _writer.println("<><>");
      _writer.flush();
      Log.log("dataclient","command",1,"DataClient.putServerDescriptor: sent all triplets");
      // Antwort lesen
      String answer=_reader.readLine();
      Log.log("dataclient","command",1,"DataClient.putServerDescriptor - returning with "+answer);
      if (answer.startsWith("ok"))
      { 
        return true;
      }
      else
      {
        // bei Bedarf Fehlercode auswerten...
      }
    }
    catch (Exception ex) 
    {
      ex.printStackTrace(Log.getWriter("dataclient","error",0));
    }
   return false;
  }
}